# nito :skull:

Lightweight components for your favorite $. Just an experiment. Or maybe not.

- For people who like $
- No templating, just $
- No virtual DOM, only $
- No JSX... $
- Just no B$


## Examples

- [Todo](https://rawgit.com/morris/nito/master/examples/todo.html)


## Getting started

Include $ first, then nito:

```
<script src="jquery-1.11.3.min.js"></script>
<script src="nito.min.js"></script>
```


## Defining Components

#### `$.nito( settings )`

- Creates a component class
- `settings` is an object that defines the prototype of the component class
- `base`, `setup`, `update`, and `idProp` are the only nito-specific properties

```
var Comp = $.nito( {

	// Base HTML for components
	// Can be a string or an array of strings
	// Arrays will be joined with \n
	// Optional, if you only use Comp.setup (see below)
	base: [
		'<div>',
			'<h1 class="title"></h1>',
		'</div>'
	],

	// Function called after creating a component
	// Optional, defaults to noop
	setup: function () {

		// the component root element
		this.$el;

		// define event handlers here
		// this.on is a shortcut to this.$el.on
		this.on( 'click', '.title', function () {

			// bound to component
			this.custom();

		} );

	},

	// Function that updates the component
	// Optional, defaults to noop
	update: function () {

		// this.find is a shortcut to this.$el.find
		this.find( '.title' ).weld( this.data.title );

	},

	// id property
	// See $el.loop below
	// Optional, defaults to 'id'
	idProp: 'id',

	// Add custom component methods here

	custom: function () {

		this.data.title = 'nito';
		this.update(); // always update explicitly

	}

} );
```


## Creating components

#### `Comp.create( data, parent )`

- Create a component using the component base HTML
- `data` will be available in the component as `this.data`. Optional
- `parent` will be available as `this.parent`. Optional
- `comp.setup()` and an initial `comp.update( data )` are called
- Returns the created component

#### `Comp.setup( base, data, parent )`

- Create a component using given base HTML or selector
- Useful for components with varying or server-rendered HTML
- See above

#### `Comp.appendTo( selector, data, parent )`

- Create a component using Comp.create and then append it to `$( selector )`
- See above


## Nesting components

Use these methods in `update`, NOT in `setup`.

#### `$el.loop( items, factory, parent )`

- For each item, create a component from factory and append to `$el`
- `items` is an array of `data` passed to the components
- Items must have distinct, truthy `id` properties, otherwise loop will throw
- `factory` should be a component class, but may be anything that has a `create` method
- Set `idProp` on factory if items have a different id property
- Existing components with same ids are reused and updated with given data
- `parent` is passed to each component. Optional
- $el must only have children generated by this loop; don't mix with more children
- Minimal DOM impact
- Returns $el

```
$el.loop( [
	{ id: 1, title: 'Write code', done: true },
	{ id: 2, title: 'Write readme', done: false }
], TodoItem );
```

#### `$el.nest( item, factory, parent )`

- Same as loop, but for one component
- Pass falsy item to not nest anything
- If item is given, needs a valid id


## Soft $ methods

These methods are soft, non-destructive, keep DOM mutations low.

#### `$set.classes( map )`

- Add classes on $set softly
- Classes not present in map are not touched
- Returns $set

```
$set.classes( { classA: truthy, classB: falsy } );
```

#### `$set.weld( data, selectors )`

- Set data on $set
- If data is not an object, set data as $set's innerHTML softly
- If data is a map of ( name: html ) pairs:
	- Will find #name or .name and set the given HTML softly
	- selectors is an optional map of ( name: selector ) pairs
	- If selectors[ name ] is given, use that instead of #name, .name
- Returns $set

```
$set.weld( 'hello' );
$set.weld( { title: 'nito', contents: 'hello' }, { contents: '.post' } );
```

#### `$set.fill( data )`

- Fill form controls contained in `$set` with given data
- Form controls must have proper `name` attributes
- Supports all controls, nested data, `name="a[b][c]"`, etc.
- Returns $set


## Router

#### $.router( handler )

- Attach a handler to the `hashchange` event
- The hash is split by `/`, empty parts are removed, and the result is passed to the handler
- The handler is also immediately called once
- Returns the (wrapped) handler

```
$.router( function ( action, id ) {

	// handle <url>#<action>/<id>

} );
```

#### $.routerOff( handler )

- Remove a handler previously returned by `$.router`
